function [ f,convergence,time,performance ] = admm_fr(b,c,I,lambda)
%ADMM for Frobenius norm of Hessian
%convergence error
tic;
epsilon = 5*10^-4;
beta = 10;
gamma = 1.618;

%setup
MAXITERS = 1000;
n = length(b(:));
dx = [0 0 0; 0 -1 1; 0 0 0]; %x direction 1st order diff kernel
dy = [0 0 0; 0 -1 0; 0 1 0]; %y direction 1st order diff kernel

%Precomputations
%2nd order gradients
Dxx = psf2otf(dx,size(b)) .* psf2otf(dx,size(b));
Dxy = sqrt(2)*psf2otf(dx,size(b)) .* psf2otf(dy,size(b));
Dyy = psf2otf(dy,size(b)) .* psf2otf(dy,size(b));

%Blur kernel OTF
C = psf2otf(c,size(b));
Ct = conj(psf2otf(c,size(b)));
%f update terms
AtA = Ct.*C;
Atg = Ct.*fft2(b);
denom = (lambda/beta)*AtA + (Dxx.*conj(Dxx) + Dxy.*conj(Dxy) + Dyy.*conj(Dyy));

%Initialize ADMM variables
f = b(:);
w1 = zeros(n,1); w2 = zeros(n,1); w3 = zeros(n,1);
u1 = zeros(n,1); u2 = zeros(n,1); u3 = zeros(n,1);
u = [u1;u2;u3];

%Initial Gradients
fxx = ifft2(fft2(reshape(f,size(b))).*Dxx);
fxx = fxx(:);
fxy = ifft2(fft2(reshape(f,size(b))).*Dxy);
fxy = fxy(:);
fyy = ifft2(fft2(reshape(f,size(b))).*Dyy);
fyy = fyy(:);

%ADMM Iterations

for iter = 1:MAXITERS
    time(iter) = toc;
    %w update
    for i = 1:n
        vi = [fxx(i);fxy(i);fyy(i)];
        ui = [u1(i);u2(i);u3(i)];
        wi = max(norm(vi + (1/beta)*ui) - 1/beta,0).*((vi + (1/beta)*ui)./norm(vi + (1/beta)*ui));
        w1(i) = wi(1);
        w2(i) = wi(2);
        w3(i) = wi(3);
    end
    w = [w1;w2;w3];
 
    %f update
    f_old = f;
    Vtw = conj(Dxx).*fft2(reshape(w1,size(b))) + conj(Dxy).*fft2(reshape(w2,size(b))) + conj(Dyy).*fft2(reshape(w3,size(b)));
    Vtu = conj(Dxx).*fft2(reshape((1/beta)*u1,size(b))) + conj(Dxy).*fft2(reshape((1/beta)*u2,size(b))) + conj(Dyy).*fft2(reshape((1/beta)*u3,size(b)));
    f = ifft2(((lambda/beta)*Atg + Vtw - Vtu)./denom);
    f = f(:);
    
    %u update
    fxx = ifft2(fft2(reshape(f,size(b))).*Dxx);
    fxx = fxx(:);
    fxy = ifft2(fft2(reshape(f,size(b))).*Dxy);
    fxy = fxy(:);
    fyy = ifft2(fft2(reshape(f,size(b))).*Dyy);
    fyy = fyy(:);
    u = u - gamma*beta*(w - [fxx;fxy;fyy]);
    u1 = u(1:n);
    u2 = u(n+1:2*n);
    u3 = u(2*n + 1:end);
    
    %performance statistics
    curr_c = norm(f - f_old)/norm(f);
    convergence(iter) = curr_c;
    performance(:,iter) = [psnr(reshape(f,size(b)),I);ssim(reshape(f,size(b)),I)];
    if(curr_c <= epsilon) break; end
end
f = reshape(f,size(b));
end