function [x,convergence,time,performance] = admm_lasso( b,c,I,lambda,normtype,kernel)
%ADMM lasso for l1 Laplacian or l1 TV regularizer. Defaults to TV

%Use 'Lap' for l1 laplacian, TV for l1 TV, Tvi for l1 TV isotropic

%Use 'Diagonals' for alternate Laplacian kernel and 'Sobel for alternate TV
%kernel

tic;
%setup
MAXITERS = 1000;
n = length(b(:));
rho = 10;
epsilon = 5*10^-4;

%Laplacian kernels
if(strcmp(kernel,'Diagonals'))
    f2 = [1 1 1; 1 -8 1; 1 1 1];
else
    f2 = [0 1 0; 1 -4 1; 0 1 0];
end
    
%TV kernels
if(strcmp(kernel,'Sobel'))
    fx = [-1 0 1; -2 0 2; -1 0 1];
    fy = [-1 -2 -1; 0 0 0; 1 2 1];
else
    fx = [0 0 0; 0 -1 1; 0 0 0];
    fy = [0 0 0; 0 -1 0; 0 1 0]; 
end

%OTFs for Fourier operations
Fx = psf2otf(fx,size(b));
Fxt = conj(psf2otf(fx,size(b)));
Fy = psf2otf(fy,size(b));
Fyt = conj(psf2otf(fy,size(b)));
F2 = psf2otf(f2,size(b));
F2t = conj(psf2otf(f2,size(b)));
C = psf2otf(c,size(b));
Ct = conj(psf2otf(c,size(b)));

%ADMM Variables
x = zeros(n,1);
if(strcmp(normtype,'Lap'))
    z = zeros(n,1);
    u = zeros(n,1);
else
    z = zeros(2*n,1);
    u = zeros(2*n,1);
end

%Precomputations
Atb = Ct.*fft2(b);
AtA =  Ct.*C;
FxtFx = Fxt.*Fx;
FytFy = Fyt.*Fy;

%Precomputes denomiator for x update
if(strcmp(normtype,'Lap'))
    denom = AtA + rho*(F2 .* F2t);
else
    denom = AtA + rho*(FxtFx + FytFy);
end

%Main ADMM Loop
for iter = 1:MAXITERS
    time(iter) = toc;
    v = z - u;
    
    %x update
    x_old = x;
    if(strcmp(normtype,'Lap'))
        x = ifft2((Atb + rho*((F2t.*fft2(reshape(v,size(b))))))./denom);
    else
        x = ifft2((Atb + rho*((Fxt.*fft2(reshape(v(1:n),size(b)))) + (Fyt.*fft2(reshape(v(n+1:end),size(b))))))./denom);
    end
    
    %z update 
    if(strcmp(normtype,'Lap'))
        D = ifft2(F2.*fft2(x));
    else
        D1 = ifft2(Fx.*fft2(x));
        D2 = ifft2(Fy.*fft2(x));
        D = [D1(:);D2(:)];
    end
    if(strcmp(normtype,'TVi'))
        D_stack = cat(3,D1,D2);
        u_stack = cat(3,reshape(u(1:n),size(b)),reshape(u(n+1:end),size(b)));
        z_stack = softTvec(u_stack + D_stack,lambda/rho);
        z = [z_stack(:,:,1),z_stack(:,:,2)];
        z = z(:);      
    else
        z = softT(u + D(:),lambda/rho);
    end
    
    %u update
    u = u + D(:) - z;
    curr_c = (norm(x(:)-x_old(:))/norm(x(:)));
    convergence(iter) = curr_c;
    performance(:,iter) = [psnr(x,I);ssim(x,I)];
    if(curr_c <= epsilon) break; end 
end
x = reshape(x,size(b));
end


